module Combine (
  Mean(..)
, Rank
, Weight
, combine
) where

import           Control.Arrow (second)
import           Data.IntMap (IntMap)
import qualified Data.IntMap as IM
import           Data.List (sort)
import           Data.Tuple (swap)

import           Types

type PredictionID = Int
type Rank         = Int
type Weight       = Double

data Mean = Arithmetic | Geometric | Quadratic | Harmonic | Maximum
  deriving (Read, Show)

meanFunction :: (Floating a, Ord a) => Mean -> [(a, a)] -> a
meanFunction Arithmetic =         sum . map (uncurry (*))
meanFunction  Geometric =         product . map (uncurry (**))
meanFunction  Quadratic =         sum . map (\ (x, w) -> x * x * w)
meanFunction   Harmonic = (1 /) . sum . map (\ (x, w) -> w / x)
meanFunction    Maximum =   maximum . map (uncurry (*))

labelRankMap :: [Labels] -> IntMap [(PredictionID, Rank)]
labelRankMap lblss =
  foldl (\ m (i, lbls) -> foldr (add i) m lbls) IM.empty zipped
  where zipped = zip [1 ..] $ map (zip [1 ..]) lblss
        add i (r, l) = IM.insertWith (++) l [(i, r)]

fillRanks :: PredictionID -> Rank -> [(PredictionID, Rank)] -> [Rank]
fillRanks maxI maxRank xs = map snd $ sort $ xs ++ remaining
  where remaining = [(i, maxRank) | i <- [1 .. maxI], i `notElem` map fst xs]

combine :: Mean -> Rank -> [Weight] -> [Labels] -> Labels
combine mean maxRank weights lblss = map snd $ sort $ map swap meaned
  where m = IM.toList $ labelRankMap lblss
        filled   = map (second $ fillRanks (length weights) maxRank) m
        weighted = map (second $ flip zip weights . map fromIntegral) filled
        meaned   = map (second $ meanFunction mean) weighted
