{-# LANGUAGE ConstraintKinds #-}

module RandomState where

import           Control.Monad (replicateM)
import           Control.Monad.State (State, StateT)
import qualified Control.Monad.State as State
import           Control.Monad.State.Class (MonadState)
import qualified Data.Sequence as Seq
import           System.Random (Random, RandomGen, randomR, split)

type RandomState  g   = State  g
type RandomStateT g m = StateT g m

type MonadRandomState = MonadState

runRandomState :: RandomGen g => RandomState g a -> g -> (a, g)
runRandomState = State.runState

runRandomStateT :: RandomGen g => RandomStateT g m a -> g -> m (a, g)
runRandomStateT = State.runStateT

-- | Monadic version of 'randomR'.
randomRM :: (RandomGen g, Random a, MonadRandomState g m) => (a, a) -> m a
randomRM range = do
  g <- State.get
  let (a, g') = randomR range g
  State.put g'
  return a

-- | Return @n@ random elements from a list, where input elements can occur
-- multiple times in the output.
-- The output for the empty list is unspecified.
sampleWithReplacement :: (RandomGen g, MonadRandomState g m) => Int -> [a] -> m [a]
sampleWithReplacement n l =
  replicateM n (randomRM (0, len - 1)) >>= return . map (Seq.index s)
  where s = Seq.fromList l; len = Seq.length s

-- | Return an infinite list of random generators.
splitMany :: RandomGen a => a -> [a]
splitMany g = g1 : splitMany g2 where (g1, g2) = split g
